@@ -22,6 +22,8 @@
 #include "configvalueprovider.h"
 #include "decorationpainter.h"
 
+constexpr int MAX_GDK_SCALE = 5;
+
 ConfigValueProvider::ConfigValueProvider()
     : kdeglobalsConfig(KSharedConfig::openConfig())
     , inputConfig(KSharedConfig::openConfig(QStringLiteral("kcminputrc")))
@@ -246,6 +248,7 @@ QMap<QString, QColor> ConfigValueProvider::colors() const
              {QStringLiteral("selection"), KCS(QPalette::Active, KCS::Selection)},
              {QStringLiteral("tooltip"), KCS(QPalette::Active, KCS::Tooltip)},
              {QStringLiteral("complementary"), KCS(QPalette::Active, KCS::Complementary)},
+             {QStringLiteral("header"), KCS(QPalette::Active, KCS::Header)},
          }},
         {QStringLiteral("inactive"),
          {
@@ -255,6 +258,7 @@ QMap<QString, QColor> ConfigValueProvider::colors() const
              {QStringLiteral("selection"), KCS(QPalette::Inactive, KCS::Selection)},
              {QStringLiteral("tooltip"), KCS(QPalette::Inactive, KCS::Tooltip)},
              {QStringLiteral("complementary"), KCS(QPalette::Inactive, KCS::Complementary)},
+             {QStringLiteral("header"), KCS(QPalette::Inactive, KCS::Header)},
          }},
         {QStringLiteral("disabled"),
          {
@@ -264,6 +268,7 @@ QMap<QString, QColor> ConfigValueProvider::colors() const
              {QStringLiteral("selection"), KCS(QPalette::Disabled, KCS::Selection)},
              {QStringLiteral("tooltip"), KCS(QPalette::Disabled, KCS::Tooltip)},
              {QStringLiteral("complementary"), KCS(QPalette::Disabled, KCS::Complementary)},
+             {QStringLiteral("header"), KCS(QPalette::Disabled, KCS::Header)},
          }},
     };
 
@@ -290,7 +295,7 @@ QMap<QString, QColor> ConfigValueProvider::colors() const
 
     KConfigGroup windowManagerConfig = kdeglobalsConfig->group(QStringLiteral("WM"));
 
-    return {
+    QMap<QString, QColor> result = {
         /*
          * Normal (Non Backdrop, Non Insensitive)
          */
@@ -410,23 +415,70 @@ QMap<QString, QColor> ConfigValueProvider::colors() const
 
         {"content_view_bg_breeze", csc["active"]["view"].background(KCS::NormalBackground).color()},
 
-        {"theme_titlebar_background_breeze", windowManagerConfig.readEntry("activeBackground", QColor())},
-        {"theme_titlebar_foreground_breeze", windowManagerConfig.readEntry("activeForeground", QColor())},
-        {"theme_titlebar_background_light_breeze", csc["active"]["window"].background(KCS::NormalBackground).color()},
-        {"theme_titlebar_foreground_backdrop_breeze", windowManagerConfig.readEntry("inactiveForeground", QColor())},
-        {"theme_titlebar_background_backdrop_breeze", windowManagerConfig.readEntry("inactiveBackground", QColor())},
-        {"theme_titlebar_foreground_insensitive_breeze", windowManagerConfig.readEntry("inactiveForeground", QColor())},
-        {"theme_titlebar_foreground_insensitive_backdrop_breeze", windowManagerConfig.readEntry("inactiveForeground", QColor())},
-
-        // Titlebar colors
-        {"theme_titlebar_background_breeze", windowManagerConfig.readEntry("activeBackground", QColor())},
-        {"theme_titlebar_foreground_breeze", windowManagerConfig.readEntry("activeForeground", QColor())},
-        {"theme_titlebar_background_light_breeze", csc["active"]["window"].background(KCS::NormalBackground).color()},
-        {"theme_titlebar_foreground_backdrop_breeze", windowManagerConfig.readEntry("inactiveForeground", QColor())},
-        {"theme_titlebar_background_backdrop_breeze", windowManagerConfig.readEntry("inactiveBackground", QColor())},
-        {"theme_titlebar_foreground_insensitive_breeze", windowManagerConfig.readEntry("inactiveForeground", QColor())},
-        {"theme_titlebar_foreground_insensitive_backdrop_breeze", windowManagerConfig.readEntry("inactiveForeground", QColor())},
     };
+    // Handle Headers (menu bars and some of toolbars)
+    if (KCS::isColorSetSupported(kdeglobalsConfig, KCS::Header)) {
+        // If we have a separate Header color set, use it for both titlebar and header coloring...
+        result.insert({{"theme_header_background_breeze", csc["active"]["header"].background().color()},
+                       {"theme_header_foreground_breeze", csc["active"]["header"].foreground().color()},
+                       {"theme_header_background_light_breeze", csc["active"]["window"].background().color()},
+                       {"theme_header_foreground_backdrop_breeze", csc["inactive"]["header"].foreground().color()},
+                       {"theme_header_background_backdrop_breeze", csc["inactive"]["header"].background().color()},
+                       {"theme_header_foreground_insensitive_breeze", csc["inactive"]["header"].foreground().color()},
+                       {"theme_header_foreground_insensitive_backdrop_breeze", csc["inactive"]["header"].foreground().color()},
+
+                       {"theme_titlebar_background_breeze", csc["active"]["header"].background().color()},
+                       {"theme_titlebar_foreground_breeze", csc["active"]["header"].foreground().color()},
+                       {"theme_titlebar_background_light_breeze", csc["active"]["window"].background().color()},
+                       {"theme_titlebar_foreground_backdrop_breeze", csc["inactive"]["header"].foreground().color()},
+                       {"theme_titlebar_background_backdrop_breeze", csc["inactive"]["header"].background().color()},
+                       {"theme_titlebar_foreground_insensitive_breeze", csc["inactive"]["header"].foreground().color()},
+                       {"theme_titlebar_foreground_insensitive_backdrop_breeze", csc["inactive"]["header"].foreground().color()}});
+    } else {
+        //... if we don't we'll use regular window colors for headerbar and WM group for a titlebar
+        result.insert({
+            {"theme_header_background_breeze", csc["active"]["window"].background().color()},
+            {"theme_header_foreground_breeze", csc["active"]["window"].foreground().color()},
+            {"theme_header_background_light_breeze", csc["active"]["window"].background().color()},
+            {"theme_header_foreground_backdrop_breeze", csc["inactive"]["window"].foreground().color()},
+            {"theme_header_background_backdrop_breeze", csc["inactive"]["window"].background().color()},
+            {"theme_header_foreground_insensitive_breeze", csc["inactive"]["window"].foreground().color()},
+            {"theme_header_foreground_insensitive_backdrop_breeze", csc["inactive"]["window"].foreground().color()},
+
+            {"theme_titlebar_background_breeze", windowManagerConfig.readEntry("activeBackground", QColor())},
+            {"theme_titlebar_foreground_breeze", windowManagerConfig.readEntry("activeForeground", QColor())},
+            {"theme_titlebar_background_light_breeze", csc["active"]["window"].background(KCS::NormalBackground).color()},
+            {"theme_titlebar_foreground_backdrop_breeze", windowManagerConfig.readEntry("inactiveForeground", QColor())},
+            {"theme_titlebar_background_backdrop_breeze", windowManagerConfig.readEntry("inactiveBackground", QColor())},
+            {"theme_titlebar_foreground_insensitive_breeze", windowManagerConfig.readEntry("inactiveForeground", QColor())},
+            {"theme_titlebar_foreground_insensitive_backdrop_breeze", windowManagerConfig.readEntry("inactiveForeground", QColor())},
+        });
+    }
+
+    return result;
+}
+
+double ConfigValueProvider::globalScaleFactor() const
+{
+    KConfigGroup configGroup = kdeglobalsConfig->group(QStringLiteral("KScreen"));
+    QString entry = configGroup.readEntry(QStringLiteral("ScaleFactor"), "1");
+    bool conversionOk = false;
+    double scaleFactor = entry.toDouble(&conversionOk);
+
+    if (conversionOk && scaleFactor > 1.0 && scaleFactor < (MAX_GDK_SCALE + 0.1))
+        return scaleFactor;
+
+    return 1.0;
+}
+
+int ConfigValueProvider::globalScaleFactorAsPercent() const
+{
+    return int(globalScaleFactor() * 100.0);
+}
+
+int ConfigValueProvider::globalScaleFactorFloor() const
+{
+    return globalScaleFactorAsPercent() / 100;
 }
 
 QString ConfigValueProvider::windowDecorationButtonsOrderInGtkNotation(const QString &kdeConfigValue) const
